@@ -48,13 +48,13 @@ struct ConnectTokenEntry {
 pub struct NetcodeServer {
     clients: Box<[Option<Connection>]>,
     pending_clients: HashMap<SocketAddr, Connection>,
-    connect_token_entries: [Option<ConnectTokenEntry>; NETCODE_MAX_CLIENTS * 4],
+    connect_token_entries: Box<[Option<ConnectTokenEntry>; NETCODE_MAX_CLIENTS * 2]>,
     protocol_id: u64,
     connect_key: [u8; NETCODE_KEY_BYTES],
     max_clients: usize,
     challenge_sequence: u64,
     challenge_key: [u8; NETCODE_KEY_BYTES],
-    address: SocketAddr,
+    public_address: SocketAddr,
     current_time: Duration,
     global_sequence: u64,
     out: [u8; NETCODE_MAX_PACKET_BYTES],
@@ -62,7 +62,6 @@ pub struct NetcodeServer {
 
 /// Result from processing an packet in the server
 #[derive(Debug, PartialEq, Eq)]
-#[allow(clippy::large_enum_variant)] // TODO: Consider boxing types
 pub enum ServerResult<'a, 's> {
     /// Nothing needs to be done.
     None,
@@ -71,18 +70,28 @@ pub enum ServerResult<'a, 's> {
     /// A payload received from the client.
     Payload(ClientID, &'a [u8]),
     /// A new client has connected
-    ClientConnected(ClientID, [u8; NETCODE_USER_DATA_BYTES], PacketToSend<'s>),
+    ClientConnected(ClientID, Box<[u8; NETCODE_USER_DATA_BYTES]>, PacketToSend<'s>),
     /// The client connection has been terminated.
     // TODO: should we return the user_data also here?
     ClientDisconnected(ClientID, Option<PacketToSend<'s>>),
 }
 
 impl NetcodeServer {
+    /// Starts a new NetcodeServer.
+    ///
+    /// # Arguments
+    ///
+    /// * `public_address` - publicly available address to which clients will attempt to connect. This is
+    /// the address used to generate the ConnectToken.
+    ///
+    /// * `protocol_id` - unique identifier for this particular game/application.
+    /// You can use a hash function with the current version of the game to generate this value
+    /// so that older versions cannot connect to newer versions.
     pub fn new(
         current_time: Duration,
         max_clients: usize,
         protocol_id: u64,
-        address: SocketAddr,
+        public_address: SocketAddr,
         private_key: [u8; NETCODE_KEY_BYTES],
     ) -> Self {
         if max_clients > NETCODE_MAX_CLIENTS {
@@ -95,7 +104,7 @@ impl NetcodeServer {
 
         Self {
             clients,
-            connect_token_entries: [None; NETCODE_MAX_CLIENTS * 4],
+            connect_token_entries: Box::new([None; NETCODE_MAX_CLIENTS * 2]),
             pending_clients: HashMap::new(),
             protocol_id,
             connect_key: private_key,
@@ -103,7 +112,7 @@ impl NetcodeServer {
             challenge_sequence: 0,
             global_sequence: 0,
             challenge_key,
-            address,
+            public_address,
             current_time,
             out: [0u8; NETCODE_MAX_PACKET_BYTES],
         }
@@ -115,7 +124,7 @@ impl NetcodeServer {
     }
 
     pub fn address(&self) -> SocketAddr {
-        self.address
+        self.public_address
     }
 
     fn find_or_add_connect_token_entry(&mut self, new_entry: ConnectTokenEntry) -> bool {
@@ -184,7 +193,7 @@ impl NetcodeServer {
 
         let connect_token = PrivateConnectToken::decode(&data, self.protocol_id, expire_timestamp, &xnonce, &self.connect_key)?;
 
-        let in_host_list = connect_token.server_addresses.iter().any(|host| *host == Some(self.address));
+        let in_host_list = connect_token.server_addresses.iter().any(|host| *host == Some(self.public_address));
         if !in_host_list {
             return Err(NetcodeError::NotInHostList);
         }
@@ -391,7 +400,7 @@ impl NetcodeServer {
                             let len = packet.encode(&mut self.out, self.protocol_id, Some((self.global_sequence, &send_key)))?;
                             self.global_sequence += 1;
                             let packet_to_send = PacketToSend::new(addr, &mut self.out[..len]);
-                            return Ok(ServerResult::ClientConnected(client_id, user_data, packet_to_send));
+                            return Ok(ServerResult::ClientConnected(client_id, Box::new(user_data), packet_to_send));
                         }
                     }
                 }
@@ -568,15 +577,19 @@ mod tests {
 
     use super::*;
 
-    #[test]
-    fn server_connection() {
-        let protocol_id = 7;
+    const TEST_KEY: &[u8; NETCODE_KEY_BYTES] = b"an example very very secret key."; // 32-bytes
+    const TEST_PROTOCOL_ID: u64 = 7;
+
+    fn new_server() -> NetcodeServer {
         let max_clients = 16;
         let server_addr = "127.0.0.1:5000".parse().unwrap();
-        let private_key = b"an example very very secret key."; // 32-bytes
-        let mut server = NetcodeServer::new(Duration::ZERO, max_clients, protocol_id, server_addr, *private_key);
+        NetcodeServer::new(Duration::ZERO, max_clients, TEST_PROTOCOL_ID, server_addr, *TEST_KEY)
+    }
 
-        let server_addresses: Vec<SocketAddr> = vec![server_addr];
+    #[test]
+    fn server_connection() {
+        let mut server = new_server();
+        let server_addresses: Vec<SocketAddr> = vec![server.address()];
         let user_data = generate_random_bytes();
         let expire_seconds = 3;
         let client_id = 4;
@@ -584,13 +597,13 @@ mod tests {
         let client_addr: SocketAddr = "127.0.0.1:3000".parse().unwrap();
         let connect_token = ConnectToken::generate(
             Duration::ZERO,
-            protocol_id,
+            TEST_PROTOCOL_ID,
             expire_seconds,
             client_id,
             timeout_seconds,
             server_addresses,
             Some(&user_data),
-            private_key,
+            TEST_KEY,
         )
         .unwrap();
         let mut client = NetcodeClient::new(Duration::ZERO, client_id, connect_token);
@@ -610,7 +623,7 @@ mod tests {
         match result {
             ServerResult::ClientConnected(r_id, r_data, PacketToSend { packet, .. }) => {
                 assert_eq!(client_id, r_id);
-                assert_eq!(user_data, r_data);
+                assert_eq!(user_data, *r_data);
                 client.process_packet(packet)
             }
             _ => unreachable!(),
@@ -662,9 +675,7 @@ mod tests {
 
     #[test]
     fn connect_token_already_used() {
-        let server_addr = "127.0.0.1:5000".parse().unwrap();
-        let private_key = b"an example very very secret key."; // 32-bytes
-        let mut server = NetcodeServer::new(Duration::ZERO, 16, 0, server_addr, *private_key);
+        let mut server = new_server();
 
         let client_addr: SocketAddr = "127.0.0.1:3000".parse().unwrap();
         let mut connect_token = ConnectTokenEntry {
